#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <ctype.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "irmp3io.h"

extern char *remote_host;
GIOChannel	*irmp3_channel;
char	bConnected = 0;


gboolean    on_irmp3_io (GIOChannel *source, GIOCondition condition, gpointer data)
{
	static char	buf[1024];
	static int	iPos = 0;
	guint		iLen;
	int		ii;
	if (condition & G_IO_IN)
	{
		g_io_channel_read(source, buf + iPos, sizeof(buf) - iPos, &iLen);
		for (ii = iPos; ii < iPos + iLen; ii++)
		{
			if ((buf[ii] == '\r') || (buf[ii] == '\n'))
				break;
		}
		if (ii < (iPos + iLen))	// end of message found
		{
			iPos = iPos + iLen;
			while (1)
			{
				// printf("Loop ii = %d, iPos = %d, iLen = %d\n", ii, iPos, iLen);
				buf[ii] = 0;
				ii++;
				irmp3_io_command(buf, data);
				while (isspace(buf[ii]) && (ii < iPos))
					ii++;
				if (ii >= iPos)
				{
					iPos = 0;
					break;
				}
				memcpy(buf, buf + ii, iPos - ii);
				iPos = iPos - ii;
				for (ii = 0; ii < iPos; ii++)
				{
					if ((buf[ii] == '\r') || (buf[ii] == '\n'))
						break;
				}
				if (ii >= iPos)
				{
					iPos = 0;
					break;
				}
			}
		}
		else
		{
			iPos = iPos + iLen;
		}
	}
	if (condition & (G_IO_ERR | G_IO_HUP | G_IO_NVAL))
	{
		printf("Error on socket...\n");
		g_io_channel_close(source);
	}

	return TRUE;
}

void
on_wMon_show                           (GtkWidget       *widget,
                                        gpointer         user_data)
{
	int	iSocket;
	int	iRetstat;
	create_m_command();
	iSocket = irmp3_io_connect(remote_host);
	if (iSocket >= 0)
	{
		GIOChannel	*pChannel;
		pChannel = g_io_channel_unix_new(iSocket);
		if (pChannel)
		{
			g_io_add_watch(pChannel, G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL, on_irmp3_io, widget);
			irmp3_channel = pChannel;
			g_io_channel_write(irmp3_channel, "query volume\r\n", sizeof("query volume\r\n"), &iRetstat);
		}
	}
}

void on_volume_changed                           (GtkWidget       *widget,
                                        gpointer         user_data)
{
	char	caBuf[1024];
	int		iLen;
	iLen = sprintf(caBuf, "mixer volume %d\r\n", (int)(GTK_ADJUSTMENT(widget)->value));
//	printf("sending: %s\n", caBuf);
	g_io_channel_write(irmp3_channel, caBuf, iLen, &iLen);
}

gboolean
on_volume_button_press_event           (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

	GtkAdjustment	*pVolumeAdj;
	pVolumeAdj = gtk_range_get_adjustment((GtkRange *)widget);
	bConnected = 1;
	gtk_signal_connect((GtkObject *)pVolumeAdj, "value-changed", on_volume_changed, NULL);
	return FALSE;
}


gboolean
on_volume_button_release_event         (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	GtkAdjustment	*pVolumeAdj;
	pVolumeAdj = gtk_range_get_adjustment((GtkRange *)widget);
	gtk_signal_disconnect_by_func((GtkObject *)pVolumeAdj, on_volume_changed, NULL);
	bConnected = 0;
	return FALSE;
}


gboolean
on_volume_property_notify_event        (GtkWidget       *widget,
                                        GdkEventProperty *event,
                                        gpointer         user_data)
{
  return FALSE;
}


void
on_play1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	char	caBuf[1024];
	int		iLen;
	iLen = sprintf(caBuf, "playlist jump 0\r\n");
	g_io_channel_write(irmp3_channel, caBuf, iLen, &iLen);
}


void
on_pause1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	char	caBuf[1024];
	int		iLen;
	iLen = sprintf(caBuf, "pause\r\n");
	g_io_channel_write(irmp3_channel, caBuf, iLen, &iLen);
}


void
on_next1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	char	caBuf[1024];
	int		iLen;
	iLen = sprintf(caBuf, "playlist jump +1\r\n");
	g_io_channel_write(irmp3_channel, caBuf, iLen, &iLen);
}


void
on_back1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	char	caBuf[1024];
	int		iLen;
	iLen = sprintf(caBuf, "playlist jump -1\r\n");
	g_io_channel_write(irmp3_channel, caBuf, iLen, &iLen);
}


void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *propertybox;
	propertybox = create_propertybox();
	gtk_widget_show(propertybox);
}



gboolean
on_table1_button_press_event           (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  return FALSE;
}


gboolean
on_wMon_button_press_event             (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	g_return_val_if_fail (widget != NULL, FALSE);
//	g_return_val_if_fail (GTK_IS_MENU (widget), FALSE);
	g_return_val_if_fail (event != NULL, FALSE);

	/* The "widget" is the menu that was supplied when
	* gtk_signal_connect_object was called.
	*/
	if (event->type == GDK_BUTTON_PRESS)
	{
		if (event->button == 3)
		{
			gtk_menu_popup((GtkMenu *)create_m_command(), NULL, NULL, NULL, NULL, event->button, event->time);
// 			gtk_menu_popup (menu, NULL, NULL, NULL, NULL, event_button->button, event_button->time);
			return TRUE;
		}
	}

	return FALSE;
}


gboolean
on_wMon_delete_event                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  g_io_channel_close(irmp3_channel);
  gtk_exit(0);
  return FALSE;
}


gboolean
on_wMon_destroy_event                  (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  return FALSE;
}


void
on_stop1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	char	caBuf[1024];
	int		iLen;
	iLen = sprintf(caBuf, "stop\r\n");
	g_io_channel_write(irmp3_channel, caBuf, iLen, &iLen);

}


void irmp3_io_callback_artist(void *data, char *text)
{
	GtkWidget	*widget = (GtkWidget *) data;
	GtkLabel	*pLabel;
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_artist");
	gtk_label_set_text(pLabel, text);
}

void irmp3_io_callback_title(void *data, char *text)
{
	GtkWidget	*widget = (GtkWidget *) data;
	GtkLabel	*pLabel;
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_title");
	gtk_label_set_text(pLabel, text);
}

void irmp3_io_callback_genre(void *data, char *text)
{
	GtkWidget	*widget = (GtkWidget *) data;
	GtkLabel	*pLabel;
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_genre");
	gtk_label_set_text(pLabel, text);
}

void irmp3_io_callback_album(void *data, char *text)
{
	GtkWidget	*widget = (GtkWidget *) data;
	GtkLabel	*pLabel;
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_album");
	gtk_label_set_text(pLabel, text);
}

void irmp3_io_callback_year(void *data, char *text)
{
	GtkWidget	*widget = (GtkWidget *) data;
	GtkLabel	*pLabel;
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_year");
	gtk_label_set_text(pLabel, text);
}

void irmp3_io_callback_comment(void *data, char *text)
{
	GtkWidget	*widget = (GtkWidget *) data;
	GtkLabel	*pLabel;
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_comment");
	gtk_label_set_text(pLabel, text);
}

void irmp3_io_callback_endofsong(void *data)
{
	GtkWidget	*widget = (GtkWidget *) data;
	GtkLabel	*pLabel;
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_title");
	gtk_label_set_text(pLabel, "none");
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_artist");
	gtk_label_set_text(pLabel, "none");
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_album");
	gtk_label_set_text(pLabel, "none");
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_year");
	gtk_label_set_text(pLabel, "none");
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_comment");
	gtk_label_set_text(pLabel, "none");
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_genre");
	gtk_label_set_text(pLabel, "none");
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_file");
	gtk_label_set_text(pLabel, "none");
}

void irmp3_io_callback_file(void *data, char *text)
{
	GtkWidget	*widget = (GtkWidget *) data;
	GtkLabel	*pLabel;
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_file");
	gtk_label_set_text(pLabel, text);
}

void irmp3_io_callback_playlist(void *data, char *text)
{
	GtkWidget	*widget = (GtkWidget *) data;
	GtkLabel	*pLabel;
	pLabel = (GtkLabel *)lookup_widget(widget, "lb_playlist");
	gtk_label_set_text(pLabel, text);
}

void irmp3_io_callback_volume(void *data, int iVolume)
{
	if (!bConnected)
	{
		GtkWidget	*widget = (GtkWidget *) data;
		GtkHScale	*pVolume;
		GtkAdjustment	*pVolumeVal;
		pVolume = (GtkHScale *)lookup_widget(widget, "volume");
		pVolumeVal = gtk_range_get_adjustment((GtkRange *)pVolume);
		gtk_adjustment_set_value(pVolumeVal, iVolume);
	}
}

void irmp3_io_callback_time(void *data, int amount_done, int amount_todo)
{
	GtkWidget	*widget = (GtkWidget *) data;
	char	caText[20];
	GtkProgressBar *pPB= (GtkProgressBar *)lookup_widget(widget, "progressbar1");
	sprintf(caText, "%d:%02d (%d:%02d)", amount_done / 60, amount_done % 60, amount_todo / 60, amount_todo % 60);
	gtk_progress_set_format_string((GtkProgress *)pPB, caText);
	gtk_progress_set_percentage((GtkProgress *)pPB, (gfloat)amount_done / (gfloat)(amount_done + amount_todo));
}

void irmp3_io_callback_beginofsong(void *data)
{
}

void irmp3_io_callback_channel(void *data, char *text, int level)
{
}

void irmp3_io_callback_mute(void *data, int status)
{
}
