#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/socket.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include "irmp3io.h"

#define LEFTALIGN 1
#define CENTERALIGN 2
#define RIGHTALIGN 4

TTF_Font *font;
void RenderStatusText(SDL_Surface* surface, TTF_Font* font, const char texttorender[255], int xpos, int ypos, int width, int fontsize, int align, int anti);

int    on_irmp3_io (int iSocket, void *data)
{
	static char	buf[1024];
	static int	iPos = 0;
	int		iLen;
	int		ii;
	iLen = read(iSocket, buf + iPos, sizeof(buf) - iPos);
	for (ii = iPos; ii < iPos + iLen; ii++)
	{
		if ((buf[ii] == '\r') || (buf[ii] == '\n'))
			break;
	}
	if (ii < (iPos + iLen))	// end of message found
	{
		iPos = iPos + iLen;
		while (1)
		{
			// printf("Loop ii = %d, iPos = %d, iLen = %d\n", ii, iPos, iLen);
			buf[ii] = 0;
			ii++;
			irmp3_io_command(buf, data);
			while (isspace(buf[ii]) && (ii < iPos))
				ii++;
			if (ii >= iPos)
			{
				iPos = 0;
				break;
			}
			memcpy(buf, buf + ii, iPos - ii);
			iPos = iPos - ii;
			for (ii = 0; ii < iPos; ii++)
			{
				if ((buf[ii] == '\r') || (buf[ii] == '\n'))
					break;
			}
			if (ii >= iPos)
			{
				iPos = 0;
				break;
			}
		}
	}
	else
	{
		iPos = iPos + iLen;
	}
	return 1;
}

int main (int argc, char *argv[])
{
	int iSocket;
	SDL_Surface *screen;
	SDL_Surface *statussurface;

	/* Initialize the SDL library */
	if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) < 0 ) {
		fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}

	/* Initialize the TTF library */
	if ( TTF_Init() < 0 ) {
			fprintf(stderr, "Couldn't initialize TTF: %s\n",SDL_GetError());
			exit(2);
	}
	atexit(TTF_Quit);

	screen = SDL_SetVideoMode(640, 480, 16, SDL_ANYFORMAT);
	if ( screen == NULL ) {
		fprintf(stderr, "Couldn't set 640x480x16 video mode: %s\n",
		SDL_GetError());
		exit(1);
	}

	//Blank the screen.
	SDL_FillRect(screen, NULL, SDL_MapRGBA(screen->format, 0, 0, 0, 0));

	statussurface = SDL_CreateRGBSurface(SDL_HWSURFACE, 640, 480, 16, 0, 0, 0, 0);

	font = TTF_OpenFont("comic.ttf", 20);

	iSocket = irmp3_io_connect(argv[1]);
	while (1)
	{
		irmp3_io_data(iSocket, statussurface);
		SDL_BlitSurface(statussurface, NULL, screen, NULL);
		SDL_Flip(screen);
	}
	

	sleep(100);

}

void irmp3_io_callback_artist(void *data, char *text)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	RenderStatusText(surface, font, text, 320, 100, 640, 20, CENTERALIGN, 1);
}

void irmp3_io_callback_title(void *data, char *text)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	RenderStatusText(surface, font, text, 320, 150, 640, 20, CENTERALIGN, 1);
}

void irmp3_io_callback_genre(void *data, char *text)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	RenderStatusText(surface, font, text, 320, 200, 640, 20, CENTERALIGN, 1);
}

void irmp3_io_callback_album(void *data, char *text)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	RenderStatusText(surface, font, text, 320, 125, 640, 20, CENTERALIGN, 1);
}

void irmp3_io_callback_year(void *data, char *text)
{
}

void irmp3_io_callback_comment(void *data, char *text)
{
}

void irmp3_io_callback_endofsong(void *data)
{
}

void irmp3_io_callback_file(void *data, char *text)
{
}

void irmp3_io_callback_playlist(void *data, char *text)
{
}

void irmp3_io_callback_volume(void *data, int iVolume)
{
}

void irmp3_io_callback_time(void *data, int amount_done, int amount_todo)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	char	caText[20];

	sprintf(caText, "%d:%02d (%d:%02d)", amount_done / 60, amount_done % 60, amount_todo / 60, amount_todo % 60);
	RenderStatusText(surface, font, caText, 320, 175, 640, 20, CENTERALIGN, 1);
	//Get the current status information so the screen isn't first drawn with dummy info.
}

void irmp3_io_callback_beginofsong(void *data)
{
}

void irmp3_io_callback_channel(void *data, char *text, int level)
{
}

void irmp3_io_callback_mute(void *data, int status)
{
}

void RenderStatusText(SDL_Surface* surface, TTF_Font* font, const char texttorender[255], int xpos, int ypos, int width, int fontsize, int align, int anti)
{
	SDL_Color* fg;
	SDL_Color* bg;
	SDL_Rect back;
	SDL_Surface* text;
	SDL_Color foregr = {255,255,255,255};
	SDL_Color backgr = {0,0,0,255};

	fg = &foregr;
	bg = &backgr;

	if (anti == 0) {
		text = TTF_RenderText_Shaded(font, texttorender, *fg, *bg);
	}
	else {
		text = TTF_RenderText_Blended(font, texttorender, *fg);
	}
	
	if (text != NULL) {

		if (align == CENTERALIGN) {
			back.x = xpos - (width/2);
		}
		else if (align == RIGHTALIGN) {
			back.x = xpos - width;
		}
		else {
			back.x = xpos;
		}
		back.y = ypos;
		back.h = text->h;
		back.w = width;
		SDL_FillRect(surface, &back, SDL_MapRGBA(surface->format, 0, 0, 0, 0)); 

		back.w = text->w;
		if (align == CENTERALIGN) {
			back.x = xpos - (text->w / 2);
		}
		else if (align == RIGHTALIGN) {
			back.x = xpos - text->w;
		}
		else {
			back.x = xpos;
		}


		SDL_BlitSurface(text, NULL, surface, &back);

		SDL_FreeSurface(text);
	}


}

//void UpdateStatus(SDL_Surface* surface)
//{
//

//		sprintf(tmptext, "%d kbps", songinfoobj->bitrate);
//		RenderStatusText(surface, font, tmptext, 320, 225, 20, CENTERALIGN, 1);
//		RenderStatusText(surface, font, statusobj->playerstatus.c_str(), 320, 300, 20, CENTERALIGN, 1);
//
//		globaltimedone = statusobj->timedone;
//		globalplayerstatus = statusobj->playerstatus;
//
//		TTF_CloseFont(font);
//}

